from functools import wraps
from typing import Any, Callable, TypeVar, cast

import grpc

from .exceptions import DiagnosticsError, DiagnosticsNotImplemented, ServiceUnavailable

TCallable = TypeVar("TCallable", bound=Callable)


def catch_errors(func: TCallable) -> TCallable:
    @wraps(func)
    async def _inner(*args: Any, **kwargs: Any) -> Any:
        try:
            return await func(*args, **kwargs)
        except grpc.RpcError as error:
            if error.code() == grpc.StatusCode.UNAVAILABLE:
                raise ServiceUnavailable from error
            if error.code() == grpc.StatusCode.UNIMPLEMENTED:
                raise DiagnosticsNotImplemented from error
            raise DiagnosticsError from error
        except Exception as error:
            raise DiagnosticsError from error

    return cast(TCallable, _inner)
